@@ -1,15 +1,15 @@
-#' MOVER confidence intervals for comparisons of independent binomial or Poisson 
+#' MOVER confidence intervals for comparisons of independent binomial or Poisson
 #' rates.
-#' 
+#'
 #' Confidence intervals applying the MOVER method ("Method of Variance Estimates
-#' Recovery", developed from the Newcombe method for binomial RD) across 
-#' different contrasts (RD, RR, OR) and distributions (binomial, Poisson) using 
+#' Recovery", developed from the Newcombe method for binomial RD) across
+#' different contrasts (RD, RR, OR) and distributions (binomial, Poisson) using
 #' equal-tailed Jeffreys intervals instead of the Wilson score method for the
 #' event rates.  Also allows more general Beta and Gamma priors for an
 #' approximate Bayesian confidence interval incorporating prior beliefs about
 #' the group event rates.
 #'
-#' @param x1,x2 Numeric vectors of numbers of events in group 1 & group 2 
+#' @param x1,x2 Numeric vectors of numbers of events in group 1 & group 2
 #'   respectively.
 #' @param n1,n2 Numeric vectors of sample sizes (for binomial rates) or exposure
 #'   times (for Poisson rates) in each group.
@@ -17,52 +17,60 @@
 #'   each group (default ai = bi = 0.5 for Jeffreys method). Gamma priors for
 #'   Poisson rates require only a1, a2.
 #' @param cc Number or logical specifying (amount of) continuity correction
-#'   (default 0).
-#' @param contrast Character string indicating the contrast required: "RD"
-#'   (default), "RR", or "OR". "p" gives an interval for the single proportion
-#'   x1/n1.
+#'   (default FALSE). Numeric value is taken as the gamma parameter in Laud 2017,
+#'   Appendix S2 (default 0.5 if cc=TRUE). Forced equal to 0.5 if type="exact".
+#'   NB: cc currently not implemented for type="wilson".
+#' @param contrast Character string indicating the contrast of interest: "RD" =
+#'   rate difference (default), "RR" = rate ratio, "OR" = odds ratio.
+#'   contrast="p" gives an interval for the single proportion x1/n1.
 #' @param type Character string indicating the method used for the intervals for
-#'   the individual group rates. "jeff" = Jeffreys equal-tailed intervals
-#'   (default), "exact" = Clopper-Pearson exact intervals (also obtained using
-#'   type = "jeff" with cc = 0.5), "wilson" = Wilson score intervals (as per
-#'   Newcombe 1998). NB: "wilson" option is included only for legacy validation
-#'   against previous published method by Newcombe. It is not recommended, as
-#'   type="jeff" achieves much better coverage properties.
-#' @param adj Logical (default FALSE) indicating whether to apply the boundary 
-#'   adjustment for Jeffreys intervals recommended on p108 of Brown et al. 
-#'   (type = "jeff" only: set to FALSE if using informative priors) 
+#'   the individual group rates.
+#'   "jeff" = Jeffreys equal-tailed intervals (default);
+#'   "exact" = Clopper-Pearson/Garwood exact intervals (note this does NOT
+#'   result in a strictly conservative interval for the contrast, except for
+#'   contrast='p'. The scoreci function with cc=TRUE is recommended as a
+#'   superior approximation of 'exact' methods);
+#'   "midp" = mid-p intervals (to be added);
+#'   "SCAS" = SCAS non-iterative intervals (to be added);
+#'   "wilson" = Wilson score intervals (as per Newcombe 1998).
+#'   NB: "wilson" option is included only for legacy validation against previous
+#'   published method by Newcombe. It is not recommended, as type="jeff"
+#'   achieves much better coverage properties.
+#' @param adj Logical (default FALSE) indicating whether to apply the boundary
+#'   adjustment for Jeffreys intervals recommended on p108 of Brown et al.
+#'   (type = "jeff" only: set to FALSE if using informative priors)
 #' @param ... Additional arguments.
 #' @inheritParams jeffreysci
 #' @importFrom stats pchisq pf pnorm pt qbeta qgamma qnorm qqnorm qt
 #' @importFrom graphics abline lines text
-#' @return A matrix containing the confidence interval for the requested contrast 
-#' @examples  
+#' @return A matrix containing the confidence interval for the requested contrast
+#' @examples
 #'   #Binomial RD, MOVER-J method:
 #'   moverci(x1 = 5, n1 = 56, x2 = 0, n2 = 29)
-#'   
+#'
 #'   #Binomial RD, Newcombe method:
 #'   moverci(x1 = 5, n1 = 56, x2 = 0, n2 = 29, type = "wilson")
 #' @author Pete Laud, \email{p.j.laud@@sheffield.ac.uk}
-#' @references 
-#'   Laud PJ. Equal-tailed confidence intervals for comparison of 
-#'   rates. Pharmaceutical Statistics [in press].
-#'   
-#'   Newcombe RG. Interval estimation for the difference between independent 
+#' @references
+#'   Laud PJ. Equal-tailed confidence intervals for comparison of
+#'   rates. Pharmaceutical Statistics 2017; 16:334-348.
+#'
+#'   Newcombe RG. Interval estimation for the difference between independent
 #'   proportions: comparison of eleven methods. Statistics in Medicine 1998;
 #;   17(8):873-890.
-#'   
+#'
 #'   Donner A, Zou G. Closed-form confidence intervals for functions of the
 #'   normal mean and standard deviation. Statistical Methods in Medical Research
 #;   2012; 21(4):347-359.
-#'   
+#'
 #'   Fagerland MW, Newcombe RG. Confidence intervals for odds ratio and relative
-#'   risk based on the inverse hyperbolic sine transformation. Statistics in  
+#'   risk based on the inverse hyperbolic sine transformation. Statistics in
 #'   Medicine 2013; 32(16):2823-2836.
-#'   
-#'   Li HQ, Tang ML, Wong WK. Confidence intervals for ratio of two poisson
-#'   rates using the method of variance estimates recovery. Computational 
+#'
+#'   Li HQ, Tang ML, Wong WK. Confidence intervals for ratio of two Poisson
+#'   rates using the method of variance estimates recovery. Computational
 #'   Statistics 2014; 29(3-4):869-889.
-#'   
+#'
 #' @export
 moverci <- function(
   x1,
@@ -73,7 +81,7 @@ moverci <- function(
   b1 = 0.5,
   a2 = 0.5,
   b2 = 0.5,
-  cc = 0,
+  cc = FALSE,
   level = 0.95,
   distrib = "bin",
   contrast = "RD",
@@ -92,7 +100,7 @@ moverci <- function(
   if (!(tolower(substr(contrast, 1, 2)) %in% c("rd", "rr", "or", "p"))) {
     print("Contrast must be one of 'RD', 'RR', 'OR' or 'p'")
     stop()
-  }   
+  }
   if (contrast != "p" && (is.null(x2) || is.null(n2))) {
     print("Argument x2 or n2 missing")
     stop()
@@ -104,7 +112,7 @@ moverci <- function(
   if (any(c(x1, n1, x2, n2) < 0)) {
     print("Negative inputs!")
     stop()
-  }	
+  }
   if (distrib == "bin" && (any(x1 > n1 + 0.001) || any(x2 > n2 + 0.001))) {
     print("x1 > n1 or x2 > n2 not possible for distrib = 'bin'")
     stop()
@@ -118,7 +126,7 @@ moverci <- function(
     stop()
   }
   if (as.character(cc) == "TRUE") cc <- 0.5
-  
+
   alpha <- 1 - level
   z <- qnorm(1 - alpha/2)
 
@@ -126,9 +134,9 @@ moverci <- function(
   # in case x1,x2 are input as vectors but n1,n2 are not
   if (length(n1) < lenx && lenx > 1) n1 <- rep(n1, length.out = lenx)
   if (length(n2) < lenx && lenx > 1) n2 <- rep(n2, length.out = lenx)
-  
+
   if (contrast == "OR" && type == "wilson") {
-    # special cases for OR handled as per Fagerland & Newcombe Table III 
+    # special cases for OR handled as per Fagerland & Newcombe Table III
     # - not needed for MOVER-J now that xi/ni is not used for p.hat
     # but still needed for legacy Wilson method
     special <- (x2 == n2) | (x1 == n1)
@@ -146,13 +154,13 @@ moverci <- function(
   }
 
   if (type %in% c("jeff", "exact")) {
-    if (type =="exact") cc <- 0.5
+    if (type == "exact") cc <- 0.5
     # MOVER-J, including optional 'continuity correction'
     j1 <- jeffreysci(x1, n1, ai = a1, bi = b1, cc = cc, level = level,
-                     distrib = distrib, adj = adj) 
+                     distrib = distrib, adj = adj)
     if (contrast != "p") {
       j2 <- jeffreysci(x2, n2, ai = a2, bi = b2, cc = cc, level = level,
-                     distrib = distrib, adj = adj) 
+                     distrib = distrib, adj = adj)
     } else j2 <- NULL
     p1hat <- j1[, 3]
     p2hat <- j2[, 3]
@@ -174,13 +182,14 @@ moverci <- function(
   if (contrast == "p") {
     lower <- l1
     upper <- u1
-    est <- j1[, 3]
+    est <- p1hat
   } else if (contrast == "RD") {
     # From Newcombe (1998), p876 "Method 10"
     lower <- p1hat - p2hat - sqrt(pmax(0, (p1hat - l1)^2 + (u2 - p2hat)^2))
     upper <- p1hat - p2hat + sqrt(pmax(0, (u1 - p1hat)^2 + (p2hat - l2)^2))
     est <-  p1hat - p2hat
-    if (adj == TRUE) { #This is optional for informative priors
+    if (adj == TRUE) {
+      #This is optional for informative priors
       lower[(x1 == 0 & x2 == n2)] <- -1
       upper[(x1 == n1 & x2 == 0)] <- 1
     }
@@ -199,11 +208,13 @@ moverci <- function(
                 U1 * L2 * (2 * q1hat - U1) * (2 * q2hat - L2)))) /
                   (L2 * (2 * q2hat - L2))
     est <- p1hat * (1 - p2hat)/(p2hat * (1 - p1hat))
-    if (adj == TRUE) { #This is optional for informative priors
+    if (adj == TRUE) {
+      #This is optional for informative priors
       upper[x2 == 0] <- Inf
       lower[(x1 == 0 & x2 == n2) | (x1 == n1 & x2 == 0)] <- 0
       upper[(x1 == 0 & x2 == n2) | (x1 == n1 & x2 == 0)] <- Inf
     }
+    if (cc > 0) lower[u2 == 1] <- 0 #Avoids LCL=NaN
   } else if (contrast == "RR") {
     # From Donner & Zou (2012), p351
     # or Li et al. (2014), p873
@@ -214,39 +225,40 @@ moverci <- function(
                u1 * (2 * p2hat - l2) * (l2 * (2 * p1hat - u1))))) /
                   (l2 * (2 * p2hat - l2))
     est <- p1hat/p2hat
-    if (adj == TRUE) { #This is optional for informative priors
+    if (adj == TRUE) {
+      #This is optional for informative priors
       upper[x2 == 0] <- Inf
       lower[(x1 == 0)] <- 0
-#      upper[(x1 == n1 & x2 == 0)] <- Inf
     }
   }
   CI <- cbind(Lower = lower, Estimate = est, Upper = upper)
   CI
 }
 
-#' Jeffreys and other approximate Bayesian confidence intervals for a single 
+#' Jeffreys and other approximate Bayesian confidence intervals for a single
 #' binomial or Poisson rate.
-#' 
-#' Generalised approximate Bayesian confidence intervals based on a Beta (for 
-#' binomial rates) or Gamma (for Poisson rates) conjugate priors. Encompassing 
+#'
+#' Generalised approximate Bayesian confidence intervals based on a Beta (for
+#' binomial rates) or Gamma (for Poisson rates) conjugate priors. Encompassing
 #' the Jeffreys method (with Beta(0.5, 0.5) or Gamma(0.5) respectively), as well
-#' as any user-specified prior distribution. Clopper-Pearson also included by 
-#' way of a "continuity correction".
-#' 
+#' as any user-specified prior distribution. Clopper-Pearson method (as
+#' quantiles of a Beta distribution as described in Brown et al. 2001) also
+#' included by way of a "continuity correction" parameter.
+#'
 #' @param x Numeric vector of number of events.
-#' @param n Numeric vector of sample sizes (for binomial rates) or exposure 
+#' @param n Numeric vector of sample sizes (for binomial rates) or exposure
 #'   times (for Poisson rates).
 #' @param ai,bi Numbers defining the Beta prior distribution (default ai = bi =
 #'   0.5 for Jeffreys interval). Gamma prior for Poisson rates requires only ai.
-#' @param cc Number or logical specifying (amount of) "continuity correction". 
+#' @param cc Number or logical specifying (amount of) "continuity correction".
 #'   cc = 0 (default) gives Jeffreys interval, cc = 0.5 gives the
-#'   Clopper-Pearson interval. A value between 0 and 0.5 allows a compromise
-#'   between proximate and conservative coverage.
+#'   Clopper-Pearson interval (or Garwood for Poisson). A value between 0 and
+#'   0.5 allows a compromise between proximate and conservative coverage.
 #' @param level Number specifying confidence level (between 0 and 1, default
 #'   0.95).
 #' @param distrib Character string indicating distribution assumed for the input
 #'   data: "bin" = binomial (default), "poi" = Poisson.
-#' @param adj Logical (default TRUE) indicating whether to apply the boundary 
+#' @param adj Logical (default TRUE) indicating whether to apply the boundary
 #'   adjustment recommended on p108 of Brown et al. (set to FALSE if informative
 #'   priors are used)
 #' @param ... Other arguments.
@@ -255,6 +267,12 @@ moverci <- function(
 #' @examples
 #'   #Jeffreys method:
 #'   jeffreysci(x = 5, n = 56)
+#' @references
+#'   Laud PJ. Equal-tailed confidence intervals for comparison of
+#'   rates. Pharmaceutical Statistics 2017; 16:334-348.
+#'
+#'   Brown LD, Cai TT, DasGupta A. Interval estimation for a binomial
+#'   proportion. Statistical Science 2001; 16(2):101-133
 #' @export
 jeffreysci <- function(
   x,
@@ -274,42 +292,43 @@ jeffreysci <- function(
   if (any(c(x, n) < 0)) {
     print("Negative inputs!")
     stop()
-  }	
+  }
   if (distrib == "bin" && (any(x > n + 0.001))) {
     print("x > n not possible for distrib = 'bin'")
     stop()
   }
   if (as.character(cc) == "TRUE") cc <- 0.5
-  
+
   alpha <- 1 - level
   if (distrib == "bin") {
-    CI.lower <- qbeta( alpha/2, x + (ai - cc), n - x + (bi + cc))
-    est <- qbeta( 0.5, x + (ai), n - x + (bi))
-    CI.upper <- qbeta(1 - alpha/2, x + (ai + cc), n - x + (bi - cc))
-    if (adj == TRUE) { #recommended adjustment at boundary values
-      CI.lower[x == 0] <- 0
-      CI.upper[x == n] <- 1
+    CI_lower <- qbeta( alpha/2, x + (ai - cc), n - x + (bi + cc))
+    est <- qbeta( 0.5, x + (ai), n - x + (bi)) #Obtain phat as the median
+    CI_upper <- qbeta(1 - alpha/2, x + (ai + cc), n - x + (bi - cc))
+    if (adj == TRUE) {
+      # adjustment at boundary values
+      CI_lower[x == 0] <- 0
+      CI_upper[x == n] <- 1
       est[x == 0] <- 0
       est[x == n] <- 1
     }
   } else if (distrib == "poi") {
     # Jeffreys prior for Poisson rate uses gamma distribution,
-    # as defined in Li et al. with "continuity correction" from Laud 2016.
-    CI.lower <- qgamma(alpha/2, x + (ai - cc), scale = 1/n)
+    # as defined in Li et al. with "continuity correction" from Laud 2017.
+    CI_lower <- qgamma(alpha/2, x + (ai - cc), scale = 1/n)
     est <- qgamma( 0.5, x + (ai), scale = 1/n)
     if (adj == TRUE) {
-      CI.lower[x == 0] <-  0
+      CI_lower[x == 0] <-  0
       est[x == 0] <-  0
-    } 
-    CI.upper <- qgamma(1 - alpha/2, (x + (ai + cc)), scale = 1/n)
+    }
+    CI_upper <- qgamma(1 - alpha/2, (x + (ai + cc)), scale = 1/n)
   }
-  CI <- cbind(Lower = CI.lower, Upper = CI.upper, est = est)
+  CI <- cbind(Lower = CI_lower, Upper = CI_upper, est = est)
   CI
 }
 
 #' Approximate Bayesian ("MOVER-B") confidence intervals for
 #' comparisons of independent binomial or Poisson rates.
-#' 
+#'
 #' Wrapper function for the MOVER-B methods.  Approximate Bayesian confidence
 #' intervals for the rate (or risk) difference ("RD") or ratio ("RR") for
 #' independent binomial or Poisson rates, or for odds ratio ("OR", binomial
@@ -317,13 +336,13 @@ jeffreysci <- function(
 #' Newcombe, and generalised as "MOVER-B" in forthcoming publication) including
 #' special case "MOVER-J" using non-informative priors with optional continuity
 #' correction.  This function is vectorised in x1, x2, n1, and n2.
-#' 
-#' @param x1,x2 Numeric vectors of numbers of events in group 1 & group 2 
+#'
+#' @param x1,x2 Numeric vectors of numbers of events in group 1 & group 2
 #'   respectively.
 #' @param n1,n2 Numeric vectors of sample sizes (for binomial rates) or exposure
 #'   times (for Poisson rates) in each group.
 #' @param a1,b1,a2,b2 Numbers defining the Beta(ai,bi) prior distributions for
-#'   each group (default ai = bi = 0.5 for Jeffreys uninformative priors). Gamma 
+#'   each group (default ai = bi = 0.5 for Jeffreys uninformative priors). Gamma
 #'   priors for Poisson rates require only a1, a2.
 #' @inheritParams moverci
 #' @export
@@ -341,7 +360,7 @@ moverbci <- function(
   level = 0.95,
   cc = 0,
   ...
-) { 
+) {
   moverci(
     x1 = x1,
     n1 = n1,
@@ -358,15 +377,15 @@ moverbci <- function(
     type = "jeff",
     adj = FALSE,
     ...
-  ) 
+  )
 }
 
 # Internal function
 quadroot <- function(a, b, c_) {
-	# GET ROOTS OF A QUADRATIC EQUATION
-	r1x <- ( - b + sqrt(b^2 - 4 * a * c_) )/(2 * a)
-	r2x <- ( - b - sqrt(b^2 - 4 * a * c_) )/(2 * a)
-	r1 <- pmin(r1x, r2x)
-	r2 <- pmax(r1x, r2x)
-	cbind(r1, r2)
-}
\ No newline at end of file
+  # GET ROOTS OF A QUADRATIC EQUATION
+  r1x <- ( - b + sqrt(b^2 - 4 * a * c_) )/(2 * a)
+  r2x <- ( - b - sqrt(b^2 - 4 * a * c_) )/(2 * a)
+  r1 <- pmin(r1x, r2x)
+  r2 <- pmax(r1x, r2x)
+  cbind(r1, r2)
+}
